﻿using System;
using System.Windows.Forms;

using AIOUSBNet;  // the namespace exposes the AIOUSB Class interface 

namespace Sample1
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
        }

        private void StartButton_Click(object sender, EventArgs e)
        {
            var numDACs = 4;
            var done = false;
            var ClockHz = 1000.0; // Hertz.  Sets 1000 output values per second (across all DACs)
            var pulseActiveDurationInMilliseconds = msNumericUpDownActive.Value;
            var pulseInactiveDurationInMilliseconds = msNumericUpDownInactive.Value;
            var waveformLengthInMillisecondsAndSamples = pulseActiveDurationInMilliseconds + pulseInactiveDurationInMilliseconds;

            UInt16 waveformBufferLength = (UInt16) (waveformLengthInMillisecondsAndSamples * numDACs);

            byte[] temp = new byte[0];
            UInt16[] SampleData = new UInt16[(int)waveformBufferLength];
            UInt16 activeCounts = (ushort)((mVnumericUpDownActive.Value / 10000 * 32768) + 32768);
            UInt16 idleCounts = (ushort)((mVnumericUpDownInactive.Value / 10000 * 32768) + 32768);
            for (uint indexIntoWaveformArray = 0; indexIntoWaveformArray < waveformLengthInMillisecondsAndSamples; indexIntoWaveformArray++)
            {
                switch (indexIntoWaveformArray % 4)
                {
                    case 0: // DAC channel 0's values
                        SampleData[indexIntoWaveformArray] = indexIntoWaveformArray < pulseActiveDurationInMilliseconds ? (UInt16)(activeCounts / 4 * 4) : idleCounts;
                        break;

                    case 1: // DAC channel 1's values
                        SampleData[indexIntoWaveformArray] = indexIntoWaveformArray < pulseActiveDurationInMilliseconds ? (UInt16) (activeCounts / 4 * 3) : idleCounts;
                        break;

                    case 2: // DAC channel 2's values
                        SampleData[indexIntoWaveformArray] = indexIntoWaveformArray < pulseActiveDurationInMilliseconds ? (UInt16)(activeCounts / 4 * 2) : idleCounts;
                        break;

                    case 3: // DAC channel 3's values
                        SampleData[indexIntoWaveformArray] = indexIntoWaveformArray < pulseActiveDurationInMilliseconds ? (UInt16)(activeCounts / 4 * 1) : idleCounts;
                        break;
                }
            }
            
            var PulsestoGenerate = PulseCountnumericUpDown1.Value;

            AIOUSB.AIOUSB_ReloadDeviceLinks();
            AIOUSB.DACSetBoardRange(AIOUSB.diOnly, 3); // set ±10V range.  Change to "2" for 0-10V range, 0 is 0-5V and 1 is ±5V //  should match HARDWARE; DOES NOT change range, only sets calibration table
            AIOUSB.GenericVendorWrite(AIOUSB.diOnly, 0xC7, 1, 256, 0, ref temp);    // buffer length 256
            AIOUSB.GenericVendorWrite(AIOUSB.diOnly, 0xC7, (UInt16)(0x0001 | (UInt16)(numDACs << 8)), 0, 0, ref temp);  // DAC Set Channel Count in upper byte of Value, lower byte = 1 

            do
            {
                uint Status = AIOUSB.DACOutputProcess(AIOUSB.diOnly, ref ClockHz, waveformBufferLength, SampleData);
                Application.DoEvents();
                if (0 == --PulsestoGenerate)
                {
                    done = true;
                }
            } while (!done);

        }
    }
}
